@@ -23,7 +23,7 @@ class ImageContentSourcePolicy(Resource):
         self.repository_digest_mirrors = repository_digest_mirrors
         super().__init__(**kwargs)
 
-    def to_dict(self):
+    def to_dict(self) -> None:
         super().to_dict()
         if not self.yaml_file:
             if not self.repository_digest_mirrors:
